/*
 * SystemClass.h
 *
 *  Created on: Feb 18, 2015
 *      Author: christian
 */

#ifndef SYSTEMCLASS_H_
#define SYSTEMCLASS_H_

#include <vector>
#include <iostream>

using namespace std;

class System
{
private:
	long double A, B, b, lambda;

public:
	int N_1, N_2;
	long double j, S;
	vector<vector<vector<long double> > > phaseMatrix;

	System(int const&,int const&,long double const&, long double const&,string const&);
	void resetSystem(int const&,int const&,long double const&, long double const&,string const&);

	long double getA(){return(this->A);};
	long double getB(){return(this->B);};
	long double get_b(){return(this->b);};
	long double getLambda(){return(this->lambda);};

	void initPhaseMatrix(string);
	void fillPhaseMatrixDoublePBC();
	void fillPhaseMatrixSinglePBC();
	void fillPhaseMatrixDoubleAPBC();
	long double calcE();
	long double calcECorr();
	void calcNewLambda();
	void solveEquations(int const&);
	long double calcSQ();

	friend long double calcLogNormCoeff(System);
	friend long double calcLogFidelity(System,System);

	friend long double modifiedB(System,int const&,int const&);
	friend long double modifiedA(System,int const&,int const&);
	friend vector<long double> modifiedAandB(System,int const&,int const&);
	friend long double fluxCorrection(System,int const&);



};


System::System(int const& N_1, int const& N_2, long double const& S, long double const& j, string const& BC)
{
	this->N_1=N_1;
	this->N_2=N_2;
	this->S=S;
	this->j=j;
	this->A=S;
	this->B=0.4L*S;
	this->b=-0.4L*S;
	this->lambda=0.0L;

	this->initPhaseMatrix(BC);

}

void System::resetSystem(int const& N_1, int const& N_2, long double const& S, long double const& j, string const& BC)
{

	this->S=S;
	this->A=S;
	this->B=0.4L*S;
	this->b=-0.4L*S;
	this->lambda=0.0L;
	this->j=j;

	if((this->N_1!=N_1)||(this->N_2!=N_2))
	{
		this->N_1=N_1;
		this->N_2=N_2;
		this->initPhaseMatrix(BC);
	}

}


#endif /* SYSTEMCLASS_H_ */
