/*
 * boundary_conditions.h
 *
 *  Created on: Jan 27, 2015
 *      Author: christian
 */

#ifndef BOUNDARY_CONDITIONS_H_
#define BOUNDARY_CONDITIONS_H_

#include <vector>
#include <cmath>
#include "SystemClass.h"

using namespace std;

void System::initPhaseMatrix(string BC)
{
	if(BC=="PBC")
		this->fillPhaseMatrixDoublePBC();
	else if(BC=="MIX")
		this->fillPhaseMatrixSinglePBC();
	else if(BC=="APBC")
		this->fillPhaseMatrixDoubleAPBC();
	else
		cout<<"Could not initialize phase matrix, wrong indicator string!"<<endl;
}

void System::fillPhaseMatrixDoublePBC()	//Fill a matrix with elements [k*d-, k*d_0, k*d+] for all k-values in 1.BZ
{
	vector<long double> phase;
	vector<vector<long double> > col;

	this->phaseMatrix.clear();

	long double f_1 = 2*M_PIl/this->N_1;	//Factor used when calculating k*d
	long double f_2 = 2*M_PIl/this->N_2;

	for(int n_1=-0.5*this->N_1; n_1<0.5*this->N_1; n_1++)
	{
		col.clear();

		for(int n_2=-0.5*this->N_2; n_2<0.5*this->N_2; n_2++)
		{
			phase.clear();

			phase.push_back(f_1*n_1-f_2*n_2); //k*d-
			phase.push_back(f_1*n_1);	//k*d_0
			phase.push_back(f_2*n_2);	//k*d+

			col.push_back(phase);
		}

		this->phaseMatrix.push_back(col);
	}

	return;
}

void System::fillPhaseMatrixSinglePBC()	//Fill a matrix with elements [k*d-, k*d_0, k*d+] for all k-values in 1.BZ
{
	vector<long double> phase;
	vector<vector<long double> > col;
	this->phaseMatrix.clear();

	long double f_1 = 2*M_PIl/this->N_1;	//Factor used when calculating k*d
	long double f_2 = 2*M_PIl/this->N_2;

	for(int n_1=-0.5*this->N_1; n_1<0.5*this->N_1; n_1++)
	{
		col.clear();

		for(int n_2=-0.5*this->N_2; n_2<0.5*this->N_2; n_2++)
		{
			phase.clear();

			phase.push_back(f_1*(n_1+0.5L)-f_2*n_2); //k*d-
			phase.push_back(f_1*(n_1+0.5L));	//k*d_0
			phase.push_back(f_2*n_2);	//k*d+

			col.push_back(phase);
		}

		this->phaseMatrix.push_back(col);
	}

	return;
}

void System::fillPhaseMatrixDoubleAPBC()	//Fill a matrix with elements [k*d-, k*d_0, k*d+] for all k-values in 1.BZ
{
	vector<long double> phase;
	vector<vector<long double> > col;
	this->phaseMatrix.clear();

	long double f_1 = 2*M_PIl/this->N_1;	//Factor used when calculating k*d
	long double f_2 = 2*M_PIl/this->N_2;

	for(int n_1=-0.5*this->N_1; n_1<0.5*this->N_1; n_1++)
	{
		col.clear();

		for(int n_2=-0.5*this->N_2; n_2<0.5*this->N_2; n_2++)
		{
			phase.clear();

			phase.push_back(f_1*(n_1+0.5L)-f_2*(n_2+0.5L)); //k*d-
			phase.push_back(f_1*(n_1+0.5L));	//k*d_0
			phase.push_back(f_2*(n_2+0.5L));	//k*d+

			col.push_back(phase);
		}

		this->phaseMatrix.push_back(col);
	}

	return;
}




#endif /* BOUNDARY_CONDITIONS_H_ */
