/*
 * fidelity.h
 *
 *  Created on: Apr 17, 2015
 *      Author: christian
 */

#ifndef FIDELITY_H_
#define FIDELITY_H_

using namespace std;

#include <vector>
#include <cmath>

#include "SystemClass.h"
#include "simplified_basic_equations.h"

long double calcLogNormCoeff(System sys)
{
	long double A=sys.getA();
	long double B=sys.getB();
	long double b=sys.get_b();
	long double lambda=sys.getLambda();

	long double gammaA, gammaB, invW;
	vector<long double> phase;

	long double logC=0.0L;

	for(int n_1=0; n_1<sys.N_1; n_1++)
	{
		for(int n_2=0; n_2<sys.N_2; n_2++)
		{
			phase=sys.phaseMatrix[n_1][n_2];

			gammaA=A*calcGA(phase);
			gammaB=B*calcGB(phase)+b*sys.j*calcGb(phase);
			invW=calcInvW(gammaA,gammaB,lambda);

			if(abs((gammaB+lambda)*invW)<pow(10.0L,-13.0L))
			{
				cout<<"jepps "<<(gammaB+lambda)*invW<<endl;
				continue;
			}


			logC+=log(1.0L+(gammaB+lambda)*invW);
		}
	}

	logC*=-0.5L;
	logC+=0.5L*sys.N_1*sys.N_2*log(2.0L);


	return(logC);
}

long double calcLogFidelity(System sys_1, System sys_2)
{
	long double A_1=sys_1.getA();
	long double B_1=sys_1.getB();
	long double b_1=sys_1.get_b();
	long double lambda_1=sys_1.getLambda();

	long double A_2=sys_2.getA();
	long double B_2=sys_2.getB();
	long double b_2=sys_2.get_b();
	long double lambda_2=sys_2.getLambda();

	long double gammaA_1, gammaB_1, invW_1, logC_1, x_1, gammaA_2, gammaB_2, invW_2, logC_2, x_2, round_sqrt;
	vector<long double> phase;

	logC_1=calcLogNormCoeff(sys_1);
	logC_2=calcLogNormCoeff(sys_2);

	long double logF=logC_1+logC_2;


	for(int n_1=0; n_1<sys_1.N_1; n_1++)
	{
		for(int n_2=0; n_2<sys_1.N_2; n_2++)
		{
			phase=sys_1.phaseMatrix[n_1][n_2];

			gammaA_1=A_1*calcGA(phase);
			gammaB_1=B_1*calcGB(phase)+b_1*sys_1.j*calcGb(phase);
			invW_1=calcInvW(gammaA_1,gammaB_1,lambda_1);
			x_1=(gammaB_1+lambda_1)*invW_1;

			gammaA_2=A_2*calcGA(phase);
			gammaB_2=B_2*calcGB(phase)+b_2*sys_2.j*calcGb(phase);
			invW_2=calcInvW(gammaA_2,gammaB_2,lambda_2);
			x_2=(gammaB_2+lambda_2)*invW_2;

			round_sqrt=(x_1-1)*(x_2-1)/((x_1+1)*(x_2+1));


			if(abs(round_sqrt)<pow(10.0L,-13.0L))
			{
				continue;
			}


			logF-=log(abs(1.0L-sgn(gammaA_1*gammaA_2)*sqrt(round_sqrt)));

		}
	}



	return(logF);
}



#endif /* FIDELITY_H_ */
