/*
 * simplified_basic_equations.h
 *
 *  Created on: Feb 5, 2015
 *      Author: christian
 */

#ifndef SIMPLIFIED_BASIC_EQUATIONS_H_
#define SIMPLIFIED_BASIC_EQUATIONS_H_

#include <cmath>
#include <fstream>
#include <vector>

using namespace std;


long double calcGA(vector<long double> const &phases)	//Calculate gammaA
{
	return(-sin(phases[0])+sin(phases[1])-sin(phases[2]));
}

long double calcGB(vector<long double> const &phases)	//Calculate gammaB
{
	return(cos(phases[0])+cos(phases[1])+cos(phases[2]));
}

long double calcGb(vector<long double> const &phases)	//Calculate gammaB
{
	return(cos(phases[1]+phases[2])+cos(phases[1]+phases[0])+cos(phases[2]-phases[0]));
}

long double calcR(long double const& m, long double const& n, System &sys)
{
	long double invN=1.0L/(sys.N_1*sys.N_2);

	long double sum=0.0L;
	vector<long double> phases;

	for(int n_1=0; n_1<sys.N_1; n_1++)
	{
		for(int n_2=0; n_2<sys.N_2; n_2++)
		{
			phases=sys.phaseMatrix[n_1][n_2];
			sum+=pow(calcGA(phases),m)*pow(calcGB(phases),n);
		}
	}

	return(sum*invN);
}

long double calcInvW(long double const &gammaA, long double const &gammaB, long double const &lambda)	//Calculate 1/w
{
	return(1/sqrt((gammaB+lambda)*(gammaB+lambda)-gammaA*gammaA));
}

long double calcW(long double const &gammaA, long double const &gammaB, long double const &lambda)	//Calculate 1/w
{
	return(sqrt((gammaB+lambda)*(gammaB+lambda)-gammaA*gammaA));
}

long double sgn(long double x)
{
	if(x<0.0L)
		return(-1.0L);
	else
		return(1.0L);
}


#endif /* SIMPLIFIED_BASIC_EQUATIONS_H_ */
