/*
 * structure_factor.h
 *
 *  Created on: Nov 11, 2014
 *      Author: christian
 */

#ifndef STRUCTURE_FACTOR_H_
#define STRUCTURE_FACTOR_H_

#include <vector>
#include <iostream>
#include <fstream>

#include "SystemClass.h"
#include "simplified_basic_equations.h"
using namespace std;


long double System::calcSQ()
{
	long double a,b;
	int q_1=-this->N_1/3;		//Q=(1/3)b_1-(1/3)b_2
	int q_2=this->N_2/3;
	long double invN=(long double)1/(this->N_1*this->N_2);
	long double gammaA, gammaB, gammaAQ, gammaBQ, invW, invWQ;
	vector<long double> phase, phaseQ;

	long double SQ=0.0L;


	for(int n_1=-this->N_1/2; n_1<this->N_1/2; n_1++)
		{
			a=n_1-q_1;

			if(a>=this->N_1/2)
				a-=this->N_1;
			else if (a<-this->N_1/2)
				a+=this->N_1;

			for(int n_2=-this->N_2/2; n_2<this->N_2/2; n_2++)
			{
				b=n_2-q_2;

				if(b>=this->N_2/2)
					b-=this->N_2;
				else if(b<-this->N_2/2)
					b+=this->N_2;



				phase=this->phaseMatrix[n_1+this->N_1/2][n_2+this->N_2/2];
				phaseQ=this->phaseMatrix[a+this->N_1/2][b+this->N_1/2];

				gammaA=this->A*calcGA(phase);
				gammaAQ=this->A*calcGA(phaseQ);
				gammaB=this->B*calcGB(phase)+this->b*this->j*calcGb(phase);
				gammaBQ=this->B*calcGB(phaseQ)+this->b*this->j*calcGb(phaseQ);
				invW=calcInvW(gammaA, gammaB, this->lambda);
				invWQ=calcInvW(gammaAQ, gammaBQ, this->lambda);

				SQ+=((gammaB+this->lambda)*(gammaBQ+this->lambda)-gammaA*gammaAQ)*invW*invWQ;

			}
		}

	SQ*=0.375*invN;
	SQ-=0.375;

	return(SQ);

}

#endif /* STRUCTURE_FACTOR_H_ */
